@@ -1,5 +1,5 @@
-# mautrix-facebook - A Matrix-Facebook Messenger puppeting bridge
-# Copyright (C) 2020 Tulir Asokan
+# mautrix-facebook - A Matrix-Facebook Messenger puppeting bridge.
+# Copyright (C) 2022 Tulir Asokan
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -13,20 +13,24 @@
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
+from __future__ import annotations
+
+from typing import Any
 import asyncio
 import logging
+import time
 
-from mautrix.types import UserID, RoomID
 from mautrix.bridge import Bridge
+from mautrix.types import RoomID, UserID
 
 from .config import Config
-from .db import init as init_db
-from .user import User, init as init_user
-from .portal import Portal, init as init_portal
-from .puppet import Puppet, init as init_puppet
+from .db import init as init_db, upgrade_table
 from .matrix import MatrixHandler
-from .context import Context
-from .version import version, linkified_version
+from .portal import Portal
+from .puppet import Puppet
+from .user import User
+from .util.interval import get_interval
+from .version import linkified_version, version
 from .web import PublicBridgeWebsite
 
 
@@ -35,32 +39,38 @@ class MessengerBridge(Bridge):
     module = "mautrix_facebook"
     command = "python -m mautrix-facebook"
     description = "A Matrix-Facebook Messenger puppeting bridge."
-    repo_url = "https://github.com/tulir/mautrix-facebook"
-    real_user_content_key = "net.maunium.facebook.puppet"
+    repo_url = "https://github.com/mautrix/facebook"
     version = version
     markdown_version = linkified_version
     config_class = Config
     matrix_class = MatrixHandler
+    upgrade_table = upgrade_table
 
     config: Config
-    public_website: PublicBridgeWebsite
+    matrix: MatrixHandler
+    public_website: PublicBridgeWebsite | None
 
     periodic_reconnect_task: asyncio.Task
 
-    def prepare_bridge(self) -> None:
+    def prepare_db(self) -> None:
+        super().prepare_db()
         init_db(self.db)
-        context = Context(az=self.az, config=self.config, loop=self.loop, bridge=self)
-        self.matrix = context.mx = MatrixHandler(context)
-        self.add_startup_actions(init_user(context))
-        init_portal(context)
-        self.add_startup_actions(init_puppet(context))
-        self._prepare_website()
-        if self.config["bridge.resend_bridge_info"]:
-            self.add_startup_actions(self.resend_bridge_info())
 
-    def _prepare_website(self) -> None:
-        self.public_website = PublicBridgeWebsite(self.config["appservice.public.shared_secret"])
-        self.az.app.add_subapp(self.config["appservice.public.prefix"], self.public_website.app)
+    def prepare_bridge(self) -> None:
+        super().prepare_bridge()
+        if self.config["appservice.public.enabled"]:
+            secret = self.config["appservice.public.shared_secret"]
+            segment_key = self.config["appservice.public.segment_key"]
+            self.public_website = PublicBridgeWebsite(
+                loop=self.loop,
+                shared_secret=secret,
+                segment_key=segment_key,
+            )
+            self.az.app.add_subapp(
+                self.config["appservice.public.prefix"], self.public_website.app
+            )
+        else:
+            self.public_website = None
 
     def prepare_stop(self) -> None:
         self.periodic_reconnect_task.cancel()
@@ -70,22 +80,25 @@ class MessengerBridge(Bridge):
         self.log.debug("Stopping facebook listeners")
         User.shutdown = True
         for user in User.by_fbid.values():
-            user.stop_listening()
-
-    def prepare_shutdown(self) -> None:
-        self.log.debug("Saving user sessions")
-        for user in User.by_mxid.values():
-            user.save()
+            user.stop_listen()
+        self.add_shutdown_actions(user.save() for user in User.by_mxid.values())
 
     async def start(self) -> None:
+        self.add_startup_actions(User.init_cls(self))
+        self.add_startup_actions(Puppet.init_cls(self))
+        Portal.init_cls(self)
+        if self.config["bridge.resend_bridge_info"]:
+            self.add_startup_actions(self.resend_bridge_info())
         await super().start()
-        self.periodic_reconnect_task = self.loop.create_task(self._try_periodic_reconnect_loop())
+        if self.public_website:
+            self.public_website.ready_wait.set_result(None)
+        self.periodic_reconnect_task = asyncio.create_task(self._try_periodic_reconnect_loop())
 
     async def resend_bridge_info(self) -> None:
         self.config["bridge.resend_bridge_info"] = False
         self.config.save()
         self.log.info("Re-sending bridge info state event to all portals")
-        for portal in Portal.all():
+        async for portal in Portal.all():
             await portal.update_bridge_info()
         self.log.info("Finished re-sending bridge info state events")
 
@@ -98,7 +111,7 @@ class MessengerBridge(Bridge):
     async def _periodic_reconnect_loop(self) -> None:
         log = logging.getLogger("mau.periodic_reconnect")
         always_reconnect = self.config["bridge.periodic_reconnect.always"]
-        interval = self.config["bridge.periodic_reconnect.interval"]
+        interval = get_interval(self.config["bridge.periodic_reconnect.interval"])
         if interval <= 0:
             log.debug("Periodic reconnection is not enabled")
             return
@@ -115,27 +128,32 @@ class MessengerBridge(Bridge):
             except asyncio.CancelledError:
                 log.debug("Periodic reconnect loop stopped")
                 return
+            must_be_connected_before = time.monotonic()
+            min_connected_time = self.config["bridge.periodic_reconnect.min_connected_time"]
+            if min_connected_time:
+                must_be_connected_before -= min_connected_time
             log.info("Executing periodic reconnections")
             for user in User.by_fbid.values():
                 if not user.is_connected and not always_reconnect:
                     log.debug("Not reconnecting %s: not connected", user.mxid)
                     continue
+                if user.is_connected and user.connection_time >= must_be_connected_before:
+                    log.debug("No reconnecting %s: connected too recently", user.mxid)
+                    continue
                 log.debug("Executing periodic reconnect for %s", user.mxid)
                 try:
                     if mode == "refresh":
                         await user.refresh()
                     elif mode == "reconnect":
-                        user.listener.disconnect()
-                        await user.listen_task
-                        user.start_listen()
+                        await user.reconnect()
                 except asyncio.CancelledError:
                     log.debug("Periodic reconnect loop stopped")
                     return
                 except Exception:
-                    log.exception("Error while reconnecting", user.mxid)
+                    log.exception("Error while reconnecting %s", user.mxid)
 
     async def get_portal(self, room_id: RoomID) -> Portal:
-        return Portal.get_by_mxid(room_id)
+        return await Portal.get_by_mxid(room_id)
 
     async def get_puppet(self, user_id: UserID, create: bool = False) -> Puppet:
         return await Puppet.get_by_mxid(user_id, create=create)
@@ -143,11 +161,22 @@ class MessengerBridge(Bridge):
     async def get_double_puppet(self, user_id: UserID) -> Puppet:
         return await Puppet.get_by_custom_mxid(user_id)
 
-    async def get_user(self, user_id: UserID) -> User:
-        return User.get_by_mxid(user_id)
+    async def get_user(self, user_id: UserID, create: bool = True) -> User:
+        return await User.get_by_mxid(user_id, create=create)
 
     def is_bridge_ghost(self, user_id: UserID) -> bool:
         return bool(Puppet.get_id_from_mxid(user_id))
 
+    async def count_logged_in_users(self) -> int:
+        return len([user for user in User.by_fbid.values() if user.fbid])
+
+    async def manhole_global_namespace(self, user_id: UserID) -> dict[str, Any]:
+        return {
+            **await super().manhole_global_namespace(user_id),
+            "User": User,
+            "Portal": Portal,
+            "Puppet": Puppet,
+        }
+
 
 MessengerBridge().run()
