@@ -1,5 +1,5 @@
-# mautrix-facebook - A Matrix-Facebook Messenger puppeting bridge
-# Copyright (C) 2020 Tulir Asokan
+# mautrix-facebook - A Matrix-Facebook Messenger puppeting bridge.
+# Copyright (C) 2022 Tulir Asokan
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -13,21 +13,34 @@
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
-from typing import Dict, Tuple, List, Any
+from __future__ import annotations
 
+from typing import Any
+import os
+
+from mautrix.bridge.config import BaseBridgeConfig
 from mautrix.types import UserID
-from mautrix.bridge.config import (BaseBridgeConfig, ConfigUpdateHelper, ForbiddenDefault,
-                                   ForbiddenKey)
+from mautrix.util.config import ConfigUpdateHelper, ForbiddenDefault, ForbiddenKey
 
 
 class Config(BaseBridgeConfig):
+    def __getitem__(self, key: str) -> Any:
+        try:
+            return os.environ[f"MAUTRIX_FACEBOOK_{key.replace('.', '_').upper()}"]
+        except KeyError:
+            return super().__getitem__(key)
+
     @property
-    def forbidden_defaults(self) -> List[ForbiddenDefault]:
+    def forbidden_defaults(self) -> list[ForbiddenDefault]:
         return [
             *super().forbidden_defaults,
-            ForbiddenDefault("appservice.public.external", "https://example.com/public",
-                             condition="appservice.public.enabled"),
-            ForbiddenDefault("bridge.permissions", ForbiddenKey("example.com"))
+            ForbiddenDefault("appservice.database", "postgres://username:password@hostname/db"),
+            ForbiddenDefault(
+                "appservice.public.external",
+                "https://example.com/public",
+                condition="appservice.public.enabled",
+            ),
+            ForbiddenDefault("bridge.permissions", ForbiddenKey("example.com")),
         ]
 
     def do_update(self, helper: ConfigUpdateHelper) -> None:
@@ -37,7 +50,8 @@ class Config(BaseBridgeConfig):
 
         copy("homeserver.asmux")
 
-        copy("appservice.community_id")
+        if self["appservice.bot_avatar"] == "mxc://maunium.net/ddtNPZSKMNqaUzqrHuWvUADv":
+            base["appservice.bot_avatar"] = "mxc://maunium.net/ygtkteZsXnGJLJHRchUwYWak"
 
         copy("appservice.public.enabled")
         copy("appservice.public.prefix")
@@ -46,6 +60,8 @@ class Config(BaseBridgeConfig):
             base["appservice.public.shared_secret"] = self._new_token()
         else:
             copy("appservice.public.shared_secret")
+        copy("appservice.public.allow_matrix_login")
+        copy("appservice.public.segment_key")
 
         copy("metrics.enabled")
         copy("metrics.listen_port")
@@ -53,23 +69,29 @@ class Config(BaseBridgeConfig):
         copy("bridge.username_template")
         copy("bridge.displayname_template")
         copy("bridge.displayname_preference")
-        copy("bridge.community_template")
         copy("bridge.command_prefix")
 
         copy("bridge.initial_chat_sync")
         copy("bridge.invite_own_puppet_to_pm")
         copy("bridge.sync_with_custom_puppets")
         copy("bridge.sync_direct_chat_list")
-        copy("bridge.login_shared_secret")
+        copy("bridge.double_puppet_server_map")
+        copy("bridge.double_puppet_allow_discovery")
+        if "bridge.login_shared_secret" in self:
+            base["bridge.login_shared_secret_map"] = {
+                base["homeserver.domain"]: self["bridge.login_shared_secret"]
+            }
+        else:
+            copy("bridge.login_shared_secret_map")
         copy("bridge.presence")
         copy("bridge.update_avatar_initial_sync")
         copy("bridge.encryption.allow")
         copy("bridge.encryption.default")
-        copy("bridge.encryption.database")
         copy("bridge.encryption.key_sharing.allow")
         copy("bridge.encryption.key_sharing.require_cross_signing")
         copy("bridge.encryption.key_sharing.require_verification")
         copy("bridge.delivery_receipts")
+        copy("bridge.federate_rooms")
         copy("bridge.allow_invites")
         copy("bridge.backfill.invite_own_puppet")
         copy("bridge.backfill.initial_limit")
@@ -82,25 +104,46 @@ class Config(BaseBridgeConfig):
             copy("bridge.periodic_reconnect.interval")
             copy("bridge.periodic_reconnect.mode")
             copy("bridge.periodic_reconnect.always")
+            copy("bridge.periodic_reconnect.min_connected_time")
         copy("bridge.resync_max_disconnected_time")
         copy("bridge.temporary_disconnect_notices")
-        copy("bridge.refresh_on_reconnection_fail")
+        copy("bridge.disable_bridge_notices")
+        if "bridge.refresh_on_reconnection_fail" in self:
+            base["bridge.on_reconnection_fail.refresh"] = self[
+                "bridge.refresh_on_reconnection_fail"
+            ]
+            base["bridge.on_reconnection_fail.wait_for"] = 0
+        else:
+            copy("bridge.on_reconnection_fail.refresh")
+            copy("bridge.on_reconnection_fail.wait_for")
         copy("bridge.resend_bridge_info")
+        copy("bridge.mute_bridging")
+        copy("bridge.tag_only_on_create")
+        copy("bridge.sandbox_media_download")
 
         copy_dict("bridge.permissions")
 
-    def _get_permissions(self, key: str) -> Tuple[bool, bool, str]:
+        for key in (
+            "bridge.periodic_reconnect.interval",
+            "bridge.on_reconnection_fail.wait_for",
+        ):
+            value = base.get(key, None)
+            if isinstance(value, list) and len(value) != 2:
+                raise ValueError(f"{key} must only be a list of two items")
+
+    def _get_permissions(self, key: str) -> tuple[bool, bool, bool, str]:
         level = self["bridge.permissions"].get(key, "")
         admin = level == "admin"
         user = level == "user" or admin
-        return user, admin, level
+        relay = level == "relay" or user
+        return relay, user, admin, level
 
-    def get_permissions(self, mxid: UserID) -> Tuple[bool, bool, str]:
+    def get_permissions(self, mxid: UserID) -> tuple[bool, bool, bool, str]:
         permissions = self["bridge.permissions"] or {}
         if mxid in permissions:
             return self._get_permissions(mxid)
 
-        homeserver = mxid[mxid.index(":") + 1:]
+        homeserver = mxid[mxid.index(":") + 1 :]
         if homeserver in permissions:
             return self._get_permissions(homeserver)
 
